import React, { useState, useRef, useEffect, createRef } from 'react';

/**
 * Custom Select Component build with accebility
 */
const Select = ({ options = [], label = 'Select a label', onSelectedOptions, renderOption, }) => {
    var _a;
    // toggle options list
    const [isOpen, setIsOpen] = useState(false);
    // state to calculate overlay position
    const [overlayTop, setOverlayTop] = useState(null);
    // selected index
    const [selectedIndex, setSelectedIndex] = useState(null);
    // define button height
    const labelRef = useRef(null);
    const handlerOptionSelected = (optionData, index) => {
        setIsOpen(!isOpen);
        onSelectedOptions && onSelectedOptions(optionData, index);
        setSelectedIndex(index);
    };
    const selectedOption = options[selectedIndex];
    useEffect(() => {
        var _a;
        // set overlay top
        setOverlayTop((((_a = labelRef.current) === null || _a === void 0 ? void 0 : _a.offsetHeight) || 0) + 10);
    }, [(_a = labelRef.current) === null || _a === void 0 ? void 0 : _a.offsetHeight]);
    // useState to add a refs to some element
    const [optionRefs, setOptionsRefs] = useState([]); // 4- create state for add li element as a ref
    // open select option according to keyboard touch
    const KEY_CODES = {
        // 1- create keycode ref
        ENTER: 'Enter',
        SPACE: 'Space',
        DOWN_ARROW: 'ArrowDown',
        UP_ARROW: 'ArrowUp',
        ESC: 'Escape',
    };
    // 8- create highlightItem
    const [highlightedIndex, setHighLightIndex] = useState(0);
    const highlightedOption = (optionIndex) => {
        setHighLightIndex(optionIndex);
    };
    const handleOnKeyDown = event => {
        // 2- handle keydown function
        // console.log(event.code);
        event.preventDefault();
        if ([KEY_CODES.SPACE].includes(event.code)) {
            setIsOpen(!isOpen);
            // 7- set focus on the list item
            highlightedOption(0);
        }
    };
    useEffect(() => {
        // 5-add empty ref when component mount
        setOptionsRefs(options.map(_ => createRef()));
    }, [options.length]); // this depences because if user add dynacally a new value to the options variable the functionamity still work
    // 9- add useEffect to get the current highlighted Ref
    useEffect(() => {
        var _a;
        if (highlightedIndex !== null && isOpen) {
            const ref = optionRefs[highlightedIndex];
            (_a = ref === null || ref === void 0 ? void 0 : ref.current) === null || _a === void 0 ? void 0 : _a.focus();
        }
    }, [isOpen, highlightedIndex]);
    console.log(optionRefs);
    /// KEYBOARD NAVIGATION  ON SELECT LIST ITEM
    const onKeyDownHandler = event => {
        console.log(event.code);
        // CLOSE SELECT DROPDOWN
        if (event.code === KEY_CODES.ESC) {
            setIsOpen(false);
            return;
        }
        // IS KEYDOWN IS ARROW KEY NAVIGATE TO THE LIST
        if (event.code === KEY_CODES.DOWN_ARROW) {
            highlightedOption(getNextOptionIndex(highlightedIndex, options));
        }
        if (event.code === KEY_CODES.UP_ARROW) {
            highlightedOption(getPrevOptionIndex(highlightedIndex, options));
        }
        // SELECT ITEM WHEN TYPING ON ENTER
        if (event.code === KEY_CODES.ENTER) {
            handlerOptionSelected(options[highlightedIndex], highlightedIndex);
        }
    };
    const getPrevOptionIndex = (currentIndex, options) => {
        if (currentIndex === null) {
            return 0;
        }
        // if last item come back to first one
        if (currentIndex === 0) {
            return options.length - 1;
        }
        // else return currentIndex + 1
        return currentIndex - 1;
    };
    const getNextOptionIndex = (currentIndex, options) => {
        if (currentIndex === null) {
            return 0;
        }
        // if last item come back to first one
        if (currentIndex === options.length - 1) {
            return 0;
        }
        // else return currentIndex + 1
        return currentIndex + 1;
    };
    return (React.createElement("div", { className: "dse-select" },
        React.createElement("button", { "data-testId": "dse-select-button", onKeyDown: handleOnKeyDown, "aria-controls": "dse-select-list", "aria-haspopup": true, "aria-expanded": isOpen ? true : undefined, ref: labelRef, className: "dse-select__label", onClick: () => setIsOpen(!isOpen) },
            React.createElement("span", null, !selectedOption ? label : selectedOption === null || selectedOption === void 0 ? void 0 : selectedOption.label),
            React.createElement("span", null, "\uD83D\uDE4C")),
        isOpen ? (React.createElement("ul", { role: "menu", id: "dse-select-list", style: { top: overlayTop }, className: "dse-select__overlay" }, options.map((option, index) => {
            const optionData = { label: option.label, value: option.value };
            const isSelected = selectedIndex === index;
            const isHighLightIndex = highlightedIndex === index;
            // 6- create ref
            const ref = optionRefs[index];
            const renderOptionsProps = {
                ref,
                option,
                isSelected,
                getOptionRecommendedProps: (overrideProps = {}) => {
                    return Object.assign({ ref, role: 'menuitemradio', 'aria-checked': isSelected ? true : undefined, 'aria-label': option === null || option === void 0 ? void 0 : option.label, tabIndex: isHighLightIndex ? -1 : 0, onKeyDown: onKeyDownHandler, onMouseEnter: () => highlightedOption(index), onMouseLeave: () => highlightedOption(null), className: `dse-select__option  ${isSelected ? 'dse-select__option--selected' : ''} ${isHighLightIndex ? 'dse-select__option--highlighted ' : ''}`, key: option.value, onClick: () => handlerOptionSelected(optionData, index) }, overrideProps);
                },
            };
            if (renderOption) {
                return renderOption(renderOptionsProps);
            }
            return (React.createElement("li", Object.assign({}, renderOptionsProps.getOptionRecommendedProps()), option.label));
        }))) : null));
};

export { Select as default };
//# sourceMappingURL=Select.js.map
